@@ -1,5 +1,5 @@
 #
-# Copyright 2013-2019 Ghent University
+# Copyright 2013-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -32,9 +32,9 @@ be used within an Easyconfig file.
 """
 import os
 import platform
-from vsc.utils import fancylogger
 
-from easybuild.tools.systemtools import get_shared_lib_ext, get_os_name, get_os_type, get_os_version
+from easybuild.base import fancylogger
+from easybuild.tools.systemtools import get_os_name, get_os_type, get_os_version
 
 
 _log = fancylogger.getLogger('easyconfig.constants', fname=False)
@@ -50,4 +50,5 @@ EASYCONFIG_CONSTANTS = {
     'OS_NAME': (get_os_name(), "System name (e.g. 'fedora' or 'RHEL')"),
     'OS_VERSION': (get_os_version(), "System version"),
     'SYS_PYTHON_VERSION': (platform.python_version(), "System Python version (platform.python_version())"),
+    'SYSTEM': ({'name': 'system', 'version': 'system'}, "System toolchain"),
 }
