@@ -37,7 +37,7 @@ import copy
 import os
 
 from easybuild.framework.easyconfig.easyconfig import resolve_template
-from easybuild.framework.easyconfig.templates import template_constant_dict
+from easybuild.framework.easyconfig.templates import TEMPLATE_NAMES_EASYBLOCK_RUN_STEP, template_constant_dict
 from easybuild.tools.build_log import EasyBuildError, raise_nosupport
 from easybuild.tools.filetools import change_dir
 from easybuild.tools.run import run_cmd
@@ -111,6 +111,10 @@ class Extension(object):
         # construct dict with template values that can be used
         self.cfg.template_values.update(template_constant_dict({'name': name, 'version': version}))
 
+        # Add install/builddir templates with values from master.
+        for name in TEMPLATE_NAMES_EASYBLOCK_RUN_STEP:
+            self.cfg.template_values[name[0]] = str(getattr(self.master, name[0], None))
+
         # list of source/patch files: we use an empty list as default value like in EasyBlock
         self.src = resolve_template(self.ext.get('src', []), self.cfg.template_values)
         self.patches = resolve_template(self.ext.get('patches', []), self.cfg.template_values)
