@@ -1,5 +1,5 @@
 ##
-# Copyright 2015-2019 Ghent University
+# Copyright 2015-2020 Ghent University
 # Copyright 2015 S3IT, University of Zurich
 #
 # This file is part of EasyBuild,
@@ -30,13 +30,12 @@ Interface for submitting jobs via GC3Pie.
 :author: Kenneth Hoste (Ghent University)
 """
 from distutils.version import LooseVersion
+import os
 from time import gmtime, strftime
 import time
 
-from pkg_resources import get_distribution, DistributionNotFound
-from vsc.utils import fancylogger
-
-from easybuild.tools.build_log import EasyBuildError, print_msg
+from easybuild.base import fancylogger
+from easybuild.tools.build_log import EasyBuildError, print_msg, print_warning
 from easybuild.tools.config import JOB_DEPS_TYPE_ABORT_ON_ERROR, JOB_DEPS_TYPE_ALWAYS_RUN, build_option
 from easybuild.tools.job.backend import JobBackend
 from easybuild.tools.utilities import only_if_module_is_available
@@ -98,15 +97,20 @@ class GC3Pie(JobBackend):
     @only_if_module_is_available('gc3libs', pkgname='gc3pie')
     def _check_version(self):
         """Check whether GC3Pie version complies with required version."""
-        try:
-            pkg = get_distribution('gc3pie')
-        except DistributionNotFound as err:
-            raise EasyBuildError(
-                "Cannot load GC3Pie package: %s" % err)
 
-        if LooseVersion(pkg.version) < LooseVersion(self.REQ_VERSION):
-            raise EasyBuildError("Found GC3Pie version %s, but version %s or more recent is required",
-                                 pkg.version, self.REQ_VERSION)
+        try:
+            from pkg_resources import get_distribution, DistributionNotFound
+            pkg = get_distribution('gc3pie')
+
+            if LooseVersion(pkg.version) < LooseVersion(self.REQ_VERSION):
+                raise EasyBuildError("Found GC3Pie version %s, but version %s or more recent is required",
+                                     pkg.version, self.REQ_VERSION)
+
+        except ImportError:
+            print_warning("Failed to check required GC3Pie version (>= %s)", self.REQ_VERSION)
+
+        except DistributionNotFound as err:
+            raise EasyBuildError("Cannot load GC3Pie package: %s", err)
 
     def init(self):
         """
@@ -190,14 +194,14 @@ class GC3Pie(JobBackend):
         if hours is None:
             hours = max_walltime
         if hours > max_walltime:
-            self.log.warn("Specified %s hours, but this is impossible. (resetting to %s hours)" % (hours, max_walltime))
+            self.log.warning("Specified %s hours, but this is impossible. (resetting to %s)" % (hours, max_walltime))
             hours = max_walltime
         named_args['requested_walltime'] = hours * hr
 
         if cores:
             named_args['requested_cores'] = cores
         else:
-            self.log.warn("Number of cores to request not specified, falling back to whatever GC3Pie does by default")
+            self.log.warning("Number of cores to request not specified, falling back to GC3Pie default")
 
         return Application(['/bin/sh', '-c', script], **named_args)
 
@@ -229,7 +233,7 @@ class GC3Pie(JobBackend):
         self._engine.retrieve_overwrites = True
 
         # some sites may not be happy with flooding the cluster with build jobs...
-        self._engine.max_in_flight = build_option('job_max_jobs')
+        self._engine.max_in_flight = build_option('job_max_jobs') or 0
 
         # Add your application to the engine. This will NOT submit
         # your application yet, but will make the engine *aware* of
@@ -251,16 +255,24 @@ class GC3Pie(JobBackend):
             self._engine.progress()
 
             # report progress
-            self._print_status_report()
+            stats = self._engine.counts(only=Application)
+            self._print_status_report(stats)
 
             # Wait a few seconds...
             time.sleep(self.poll_interval)
 
         # final status report
         print_msg("Done processing jobs", log=self.log, silent=build_option('silent'))
-        self._print_status_report()
+        self._print_status_report(stats)
 
-    def _print_status_report(self):
+        # fail if at least one job has failed
+        if stats['failed'] > 0:
+            error_msg = "%d jobs failed: %s" % (stats['failed'], ', '.join(self._list_failed_jobs()))
+            raise EasyBuildError(error_msg)
+        else:
+            return os.EX_OK
+
+    def _print_status_report(self, stats):
         """
         Print a job status report to STDOUT and the log file.
 
@@ -268,7 +280,17 @@ class GC3Pie(JobBackend):
         figures are extracted from the `counts()` method of the
         currently-running GC3Pie engine.
         """
-        stats = self._engine.counts(only=Application)
-        states = ', '.join(["%d %s" % (stats[s], s.lower()) for s in stats if s != 'total' and stats[s]])
+        states = ', '.join(["%d %s" % (stats[s], s.lower()) for s in stats
+                            if s != 'total' and stats[s]])
         print_msg("GC3Pie job overview: %s (total: %s)" % (states, self.job_cnt),
                   log=self.log, silent=build_option('silent'))
+
+    def _list_failed_jobs(self):
+        """
+        Return list of names of failed build jobs.
+        """
+        failed = []
+        for job in self._engine.iter_tasks(only_cls=Application):
+            if job.execution.returncode != 0:
+                failed.append(job.name)
+        return failed
