@@ -1,5 +1,5 @@
 ##
-# Copyright 2012-2019 Ghent University
+# Copyright 2012-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -33,8 +33,8 @@ from distutils.version import LooseVersion
 import os
 import re
 import tempfile
-from vsc.utils import fancylogger
 
+from easybuild.base import fancylogger
 from easybuild.tools.build_log import EasyBuildError, print_msg
 from easybuild.tools.config import JOB_DEPS_TYPE_ABORT_ON_ERROR, JOB_DEPS_TYPE_ALWAYS_RUN, build_option
 from easybuild.tools.job.backend import JobBackend
@@ -217,7 +217,7 @@ class PbsJob(object):
         if hours is None:
             hours = max_walltime
         if hours > max_walltime:
-            self.log.warn("Specified %s hours, but this is impossible. (resetting to %s hours)" % (hours, max_walltime))
+            self.log.warning("Specified %s hours, but this is impossible. (resetting to %s)" % (hours, max_walltime))
             hours = max_walltime
 
         if ppn is None:
@@ -227,7 +227,7 @@ class PbsJob(object):
         if cores is None:
             cores = max_cores
         if cores > max_cores:
-            self.log.warn("number of requested cores (%s) was greater than available (%s) " % (cores, max_cores))
+            self.log.warning("number of requested cores (%s) was greater than available (%s) " % (cores, max_cores))
             cores = max_cores
 
         # only allow cores and hours for now.
