@@ -51,7 +51,7 @@ from easybuild.tools.module_generator import module_generator
 from easybuild.tools.modules import reset_module_caches
 from easybuild.tools.utilities import time2str
 from easybuild.tools.version import get_git_revision, this_is_easybuild
-
+from easybuild.tools.py2vs3 import string_type
 
 class EasyBlockTest(EnhancedTestCase):
     """ Baseclass for easyblock testcases """
@@ -318,11 +318,10 @@ class EasyBlockTest(EnhancedTestCase):
         os.makedirs(eb.installdir)
         open(os.path.join(eb.installdir, 'foo.jar'), 'w').write('foo.jar')
         open(os.path.join(eb.installdir, 'bla.jar'), 'w').write('bla.jar')
-        os.mkdir(os.path.join(eb.installdir, 'bin'))
-        os.mkdir(os.path.join(eb.installdir, 'bin', 'testdir'))
-        os.mkdir(os.path.join(eb.installdir, 'sbin'))
-        os.mkdir(os.path.join(eb.installdir, 'share'))
-        os.mkdir(os.path.join(eb.installdir, 'share', 'man'))
+        for path in ('bin', ('bin', 'testdir'), 'sbin', 'share', ('share', 'man'), 'lib', 'lib64'):
+            if isinstance(path, string_type):
+                path = (path, )
+            os.mkdir(os.path.join(eb.installdir, *path))
         # this is not a path that should be picked up
         os.mkdir(os.path.join(eb.installdir, 'CPATH'))
 
@@ -332,6 +331,7 @@ class EasyBlockTest(EnhancedTestCase):
             self.assertTrue(re.search(r"^prepend-path\s+CLASSPATH\s+\$root/bla.jar$", guess, re.M))
             self.assertTrue(re.search(r"^prepend-path\s+CLASSPATH\s+\$root/foo.jar$", guess, re.M))
             self.assertTrue(re.search(r"^prepend-path\s+MANPATH\s+\$root/share/man$", guess, re.M))
+            self.assertTrue(re.search(r"^prepend-path\s+CMAKE_PREFIX_PATH\s+\$root$", guess, re.M))
             # bin/ is not added to $PATH if it doesn't include files
             self.assertFalse(re.search(r"^prepend-path\s+PATH\s+\$root/bin$", guess, re.M))
             self.assertFalse(re.search(r"^prepend-path\s+PATH\s+\$root/sbin$", guess, re.M))
@@ -341,6 +341,7 @@ class EasyBlockTest(EnhancedTestCase):
             self.assertTrue(re.search(r'^prepend_path\("CLASSPATH", pathJoin\(root, "bla.jar"\)\)$', guess, re.M))
             self.assertTrue(re.search(r'^prepend_path\("CLASSPATH", pathJoin\(root, "foo.jar"\)\)$', guess, re.M))
             self.assertTrue(re.search(r'^prepend_path\("MANPATH", pathJoin\(root, "share/man"\)\)$', guess, re.M))
+            self.assertTrue('prepend_path("CMAKE_PREFIX_PATH", root)' in guess)
             # bin/ is not added to $PATH if it doesn't include files
             self.assertFalse(re.search(r'^prepend_path\("PATH", pathJoin\(root, "bin"\)\)$', guess, re.M))
             self.assertFalse(re.search(r'^prepend_path\("PATH", pathJoin\(root, "sbin"\)\)$', guess, re.M))
@@ -361,6 +362,41 @@ class EasyBlockTest(EnhancedTestCase):
         else:
             self.assertTrue(False, "Unknown module syntax: %s" % get_module_syntax())
 
+        # Check that lib64 is only added to CMAKE_LIBRARY_PATH if there are files in there
+        # but only if it is not a symlink to lib
+        # -- No Files
+        if get_module_syntax() == 'Tcl':
+            self.assertFalse(re.search(r"^prepend-path\s+CMAKE_LIBRARY_PATH\s+\$root/lib64$", guess, re.M))
+        elif get_module_syntax() == 'Lua':
+            self.assertFalse('prepend_path("CMAKE_LIBRARY_PATH", pathJoin(root, "lib64"))' in guess)
+        # -- With files
+        open(os.path.join(eb.installdir, 'lib64', 'libfoo.so'), 'w').write('test')
+        guess = eb.make_module_req()
+        if get_module_syntax() == 'Tcl':
+            self.assertTrue(re.search(r"^prepend-path\s+CMAKE_LIBRARY_PATH\s+\$root/lib64$", guess, re.M))
+        elif get_module_syntax() == 'Lua':
+            self.assertTrue('prepend_path("CMAKE_LIBRARY_PATH", pathJoin(root, "lib64"))' in guess)
+        # -- With files in lib and lib64 symlinks to lib
+        open(os.path.join(eb.installdir, 'lib', 'libfoo.so'), 'w').write('test')
+        shutil.rmtree(os.path.join(eb.installdir, 'lib64'))
+        os.symlink('lib', os.path.join(eb.installdir, 'lib64'))
+        guess = eb.make_module_req()
+        if get_module_syntax() == 'Tcl':
+            self.assertFalse(re.search(r"^prepend-path\s+CMAKE_LIBRARY_PATH\s+\$root/lib64$", guess, re.M))
+        elif get_module_syntax() == 'Lua':
+            self.assertFalse('prepend_path("CMAKE_LIBRARY_PATH", pathJoin(root, "lib64"))' in guess)
+
+        # With files in /lib and /lib64 symlinked to /lib there should be exactly 1 entry for (LD_)LIBRARY_PATH
+        # pointing to /lib
+        for var in ('LIBRARY_PATH', 'LD_LIBRARY_PATH'):
+            if get_module_syntax() == 'Tcl':
+                self.assertFalse(re.search(r"^prepend-path\s+%s\s+\$root/lib64$" % var, guess, re.M))
+                self.assertEqual(len(re.findall(r"^prepend-path\s+%s\s+\$root/lib$" % var, guess, re.M)), 1)
+            elif get_module_syntax() == 'Lua':
+                self.assertFalse(re.search(r'^prepend_path\("%s", pathJoin\(root, "lib64"\)\)$' % var, guess, re.M))
+                self.assertEqual(len(re.findall(r'^prepend_path\("%s", pathJoin\(root, "lib"\)\)$' % var,
+                                                guess, re.M)), 1)
+
         # check for behavior when a string value is used as dict value by make_module_req_guesses
         eb.make_module_req_guess = lambda: {'PATH': 'bin'}
         txt = eb.make_module_req()
@@ -786,7 +822,6 @@ class EasyBlockTest(EnhancedTestCase):
 
     def test_skip_extensions_step(self):
         """Test the skip_extensions_step"""
-        init_config(build_options={'silent': True})
 
         self.contents = cleandoc("""
             easyblock = "ConfigureMake"
@@ -797,11 +832,12 @@ class EasyBlockTest(EnhancedTestCase):
             toolchain = SYSTEM
             exts_list = [
                 "ext1",
-                ("ext2", "42", {"source_tmpl": "dummy.tgz"}),
+                ("EXT-2", "42", {"source_tmpl": "dummy.tgz"}),
                 ("ext3", "1.1", {"source_tmpl": "dummy.tgz", "modulename": "real_ext"}),
+                "ext4",
             ]
             exts_filter = ("\
-                if [ %(ext_name)s == 'ext2' ] && [ %(ext_version)s == '42' ] && [[ %(src)s == *dummy.tgz ]];\
+                if [ %(ext_name)s == 'ext_2' ] && [ %(ext_version)s == '42' ] && [[ %(src)s == *dummy.tgz ]];\
                     then exit 0;\
                 elif [ %(ext_name)s == 'real_ext' ]; then exit 0;\
                 else exit 1; fi", "")
@@ -813,12 +849,30 @@ class EasyBlockTest(EnhancedTestCase):
         eb.builddir = config.build_path()
         eb.installdir = config.install_path()
         eb.skip = True
+
+        self.mock_stdout(True)
         eb.extensions_step(fetch=True)
-        # 'ext1' should be in eb.exts
-        eb_exts = [y for x in eb.exts for y in x.values()]
+        stdout = self.get_stdout()
+        self.mock_stdout(False)
+
+        patterns = [
+            r"^== skipping extension EXT-2",
+            r"^== skipping extension ext3",
+            r"^== installing extension ext1  \(1/2\)\.\.\.",
+            r"^== installing extension ext4  \(2/2\)\.\.\.",
+        ]
+        for pattern in patterns:
+            regex = re.compile(pattern, re.M)
+            self.assertTrue(regex.search(stdout), "Pattern '%s' found in: %s" % (regex.pattern, stdout))
+
+        # 'ext1' should be in eb.ext_instances
+        eb_exts = [x.name for x in eb.ext_instances]
         self.assertTrue('ext1' in eb_exts)
-        # 'ext2' should not
-        self.assertFalse('ext2' in eb_exts)
+        # 'EXT-2' should not
+        self.assertFalse('EXT-2' in eb_exts)
+        self.assertFalse('EXT_2' in eb_exts)
+        self.assertFalse('ext-2' in eb_exts)
+        self.assertFalse('ext_2' in eb_exts)
         # 'ext3' should not
         self.assertFalse('ext3' in eb_exts)
 
@@ -1711,11 +1765,11 @@ class EasyBlockTest(EnhancedTestCase):
 
         # full check also catches checksum issues with extensions
         res = eb.check_checksums()
-        self.assertEqual(len(res), 5)
+        self.assertEqual(len(res), 4)
         run_checks()
 
         idx = 2
-        for ext in ['bar', 'barbar', 'toy']:
+        for ext in ['bar', 'barbar']:
             expected = "Checksums missing for one or more sources/patches of extension %s in " % ext
             self.assertTrue(res[idx].startswith(expected))
             idx += 1
