@@ -1,5 +1,5 @@
 # #
-# Copyright 2013-2019 Ghent University
+# Copyright 2013-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -31,7 +31,6 @@ import os
 import sys
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered
 from unittest import TextTestRunner
-from vsc.utils.fancylogger import setLogLevelDebug, logToScreen
 
 import easybuild.tools.build_log
 from easybuild.framework.easyconfig.format.format import Dependency
@@ -40,6 +39,7 @@ from easybuild.framework.easyconfig.format.version import EasyVersion
 from easybuild.framework.easyconfig.parser import EasyConfigParser
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.filetools import read_file
+from easybuild.tools.py2vs3 import string_type
 
 
 TESTDIRBASE = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs')
@@ -55,7 +55,7 @@ class EasyConfigParserTest(EnhancedTestCase):
 
         ec = ecp.get_config_dict()
 
-        self.assertEqual(ec['toolchain'], {'name': 'dummy', 'version': 'dummy'})
+        self.assertEqual(ec['toolchain'], {'name': 'system', 'version': 'system'})
         self.assertEqual(ec['name'], 'GCC')
         self.assertEqual(ec['version'], '4.6.3')
 
@@ -77,7 +77,7 @@ class EasyConfigParserTest(EnhancedTestCase):
 
         # this should be ok: ie the default values
         ec = ecp.get_config_dict()
-        self.assertEqual(ec['toolchain'], {'name': 'dummy', 'version': 'dummy'})
+        self.assertEqual(ec['toolchain'], {'name': 'system', 'version': 'system'})
         self.assertEqual(ec['name'], 'GCC')
         self.assertEqual(ec['version'], '4.6.2')
 
@@ -130,7 +130,7 @@ class EasyConfigParserTest(EnhancedTestCase):
         ec = ecp.get_config_dict()
         self.assertEqual(ec['name'], 'foss')
         self.assertEqual(ec['version'], '2018a')
-        self.assertEqual(ec['toolchain'], {'name': 'dummy', 'version': 'dummy'})
+        self.assertEqual(ec['toolchain'], {'name': 'system', 'version': 'system'})
 
         # dependencies should be parsed correctly
         deps = [
@@ -175,11 +175,16 @@ class EasyConfigParserTest(EnhancedTestCase):
     def test_easyconfig_constants(self):
         """Test available easyconfig constants."""
         constants = build_easyconfig_constants_dict()
+
+        # SYSTEM constant is a dict value, so takes special care
+        system_constant = constants.pop('SYSTEM')
+        self.assertEqual(system_constant, {'name': 'system', 'version': 'system'})
+
         # make sure both keys and values are only strings
         for constant_name in constants:
-            self.assertTrue(isinstance(constant_name, basestring), "Constant name %s is a string" % constant_name)
+            self.assertTrue(isinstance(constant_name, string_type), "Constant name %s is a string" % constant_name)
             val = constants[constant_name]
-            self.assertTrue(isinstance(val, basestring), "Constant value %s is a string" % val)
+            self.assertTrue(isinstance(val, (string_type, dict)), "Constant value %s is a string or dict" % val)
 
         # check a couple of randomly picked constant values
         self.assertEqual(constants['SOURCE_TAR_GZ'], '%(name)s-%(version)s.tar.gz')
@@ -206,6 +211,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    # logToScreen(enable=True)
-    # setLogLevelDebug()
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
