@@ -1,5 +1,5 @@
 # #
-# Copyright 2014-2019 Ghent University
+# Copyright 2014-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -34,10 +34,11 @@ import sys
 from test.framework.utilities import EnhancedTestCase, TestLoaderFiltered, init_config
 from unittest import TextTestRunner
 
-import easybuild.tools.job.slurm as slurm
 from easybuild.framework.easyconfig.tools import process_easyconfig
 from easybuild.tools import config
-from easybuild.tools.filetools import adjust_permissions, mkdir, remove_dir, which, write_file
+from easybuild.tools.build_log import EasyBuildError
+from easybuild.tools.config import get_module_syntax
+from easybuild.tools.filetools import adjust_permissions, mkdir, read_file, remove_dir, which, write_file
 from easybuild.tools.job import pbs_python
 from easybuild.tools.job.pbs_python import PbsPython
 from easybuild.tools.options import parse_options
@@ -214,7 +215,7 @@ class ParallelBuildTest(EnhancedTestCase):
         try:
             import gc3libs  # noqa (ignore unused import)
         except ImportError:
-            print "GC3Pie not available, skipping test"
+            print("GC3Pie not available, skipping test")
             return
 
         # put GC3Pie config in place to use local host and fork/exec
@@ -258,8 +259,23 @@ class ParallelBuildTest(EnhancedTestCase):
         cmd = "PYTHONPATH=%s:%s:$PYTHONPATH eb %%(spec)s -df" % (topdir, test_easyblocks_path)
         build_easyconfigs_in_parallel(cmd, ordered_ecs, prepare_first=False)
 
-        self.assertTrue(os.path.join(self.test_installpath, 'modules', 'all', 'toy', '0.0'))
-        self.assertTrue(os.path.join(self.test_installpath, 'software', 'toy', '0.0', 'bin', 'toy'))
+        toy_modfile = os.path.join(self.test_installpath, 'modules', 'all', 'toy', '0.0')
+        if get_module_syntax() == 'Lua':
+            toy_modfile += '.lua'
+        self.assertTrue(os.path.exists(toy_modfile))
+        self.assertTrue(os.path.exists(os.path.join(self.test_installpath, 'software', 'toy', '0.0', 'bin', 'toy')))
+
+        # also check what happens when a job fails (an error should be raised)
+        test_ecfile = os.path.join(self.test_prefix, 'test.eb')
+        ectxt = read_file(ec_file)
+        # use different version, for which no sources are available
+        regex = re.compile('^version = .*', re.M)
+        ectxt = regex.sub("version = '1.2.3'", ectxt)
+        write_file(test_ecfile, ectxt)
+        ecs = resolve_dependencies(process_easyconfig(test_ecfile), self.modtool)
+
+        error = "1 jobs failed: toy-1.2.3"
+        self.assertErrorRegex(EasyBuildError, error, build_easyconfigs_in_parallel, cmd, ecs, prepare_first=False)
 
     def test_submit_jobs(self):
         """Test submit_jobs"""
@@ -348,8 +364,7 @@ class ParallelBuildTest(EnhancedTestCase):
             'job-name': 'gzip-1.5-foss-2018a',
             'nodes': 1,
             'ntasks': 3,
-            'ntasks-per-node': 3,
-            'output': '%x-%j.out',
+            'output': 'gzip-1.5-foss-2018a-%j.out',
             'time': 300,  # 60*5 (unit is minutes)
             'wrap': "echo '%s'" % test_ec,
         }
@@ -362,4 +377,5 @@ def suite():
 
 
 if __name__ == '__main__':
-    TextTestRunner(verbosity=1).run(suite())
+    res = TextTestRunner(verbosity=1).run(suite())
+    sys.exit(len(res.failures))
