@@ -1,5 +1,5 @@
 ##
-# Copyright 2009-2019 Ghent University
+# Copyright 2009-2020 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -32,6 +32,7 @@ import os
 import platform
 import shutil
 
+from easybuild.framework.easyconfig import CUSTOM
 from easybuild.framework.extensioneasyblock import ExtensionEasyBlock
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.environment import setvar
@@ -43,6 +44,16 @@ from easybuild.tools.run import run_cmd
 class EB_toy(ExtensionEasyBlock):
     """Support for building/installing toy."""
 
+    @staticmethod
+    def extra_options(extra_vars=None):
+        """Custom easyconfig parameters for toytoy."""
+        if extra_vars is None:
+            extra_vars = {}
+
+        extra_vars['make_module'] = [True, "Skip generating (final) module file", CUSTOM]
+
+        return ExtensionEasyBlock.extra_options(extra_vars)
+
     def __init__(self, *args, **kwargs):
         """Constructor"""
         super(EB_toy, self).__init__(*args, **kwargs)
@@ -57,6 +68,16 @@ class EB_toy(ExtensionEasyBlock):
         self.cfg['exts_defaultclass'] = "Toy_Extension"
         self.cfg['exts_filter'] = ("%(ext_name)s", "")
 
+    def run_all_steps(self, *args, **kwargs):
+        """
+        Tweak iterative easyconfig parameters.
+        """
+        if isinstance(self.cfg['buildopts'], list):
+            # inject list of values for prebuildopts, same length as buildopts
+            self.cfg['prebuildopts'] = ["echo hello && "] * len(self.cfg['buildopts'])
+
+        return super(EB_toy, self).run_all_steps(*args, **kwargs)
+
     def configure_step(self, name=None):
         """Configure build of toy."""
         if name is None:
@@ -106,6 +127,15 @@ class EB_toy(ExtensionEasyBlock):
         self.build_step()
         self.install_step()
 
+    def make_module_step(self, fake=False):
+        """Generate module file."""
+        if self.cfg.get('make_module', True) or fake:
+            modpath = super(EB_toy, self).make_module_step(fake=fake)
+        else:
+            modpath = self.module_generator.get_modules_path(fake=fake)
+
+        return modpath
+
     def make_module_extra(self):
         """Extra stuff for toy module"""
         txt = super(EB_toy, self).make_module_extra()
