@@ -36,8 +36,10 @@ from easybuild.framework.easyconfig.easyconfig import get_toolchain_hierarchy, p
 from easybuild.framework.easyconfig.parser import EasyConfigParser
 from easybuild.framework.easyconfig.tweak import find_matching_easyconfigs, obtain_ec_for, pick_version, tweak_one
 from easybuild.framework.easyconfig.tweak import check_capability_mapping, match_minimum_tc_specs
-from easybuild.framework.easyconfig.tweak import get_dep_tree_of_toolchain
-from easybuild.framework.easyconfig.tweak import map_toolchain_hierarchies, map_easyconfig_to_target_tc_hierarchy
+from easybuild.framework.easyconfig.tweak import get_dep_tree_of_toolchain, map_common_versionsuffixes
+from easybuild.framework.easyconfig.tweak import get_matching_easyconfig_candidates, map_toolchain_hierarchies
+from easybuild.framework.easyconfig.tweak import find_potential_version_mappings
+from easybuild.framework.easyconfig.tweak import map_easyconfig_to_target_tc_hierarchy
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.config import module_classes
 from easybuild.tools.filetools import change_dir, write_file
@@ -305,16 +307,90 @@ class TweakTest(EnhancedTestCase):
         }
         self.assertEqual(map_toolchain_hierarchies(gcc_binutils_tc, iccifort_binutils_tc, self.modtool), expected)
 
-    def test_map_easyconfig_to_target_tc_hierarchy(self):
-        """Test mapping of easyconfig to target hierarchy"""
+    def test_get_matching_easyconfig_candidates(self):
+        """Test searching for easyconfig candidates based on a stub and toolchain"""
         test_easyconfigs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
         init_config(build_options={
-            'robot_path': test_easyconfigs,
+            'valid_module_classes': module_classes(),
+            'robot_path': [test_easyconfigs],
+        })
+        toolchain = {'name': 'GCC', 'version': '4.9.3-2.26'}
+        paths, toolchain_suff = get_matching_easyconfig_candidates('gzip-', toolchain)
+        expected_toolchain_suff = '-GCC-4.9.3-2.26'
+        self.assertEqual(toolchain_suff, expected_toolchain_suff)
+        expected_paths = [os.path.join(test_easyconfigs, 'g', 'gzip', 'gzip-1.4' + expected_toolchain_suff + '.eb')]
+        self.assertEqual(paths, expected_paths)
+
+        paths, toolchain_stub = get_matching_easyconfig_candidates('nosuchmatch', toolchain)
+        self.assertEqual(paths, [])
+        self.assertEqual(toolchain_stub, expected_toolchain_suff)
+
+    def test_map_common_versionsuffixes(self):
+        """Test mapping between two toolchain hierarchies"""
+        test_easyconfigs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
+        init_config(build_options={
+            'robot_path': [test_easyconfigs],
             'silent': True,
             'valid_module_classes': module_classes(),
         })
         get_toolchain_hierarchy.clear()
 
+        gcc_binutils_tc = {'name': 'GCC', 'version': '4.9.3-2.26'}
+        iccifort_binutils_tc = {'name': 'iccifort', 'version': '2016.1.150-GCC-4.9.3-2.25'}
+
+        toolchain_mapping = map_toolchain_hierarchies(iccifort_binutils_tc, gcc_binutils_tc, self.modtool)
+        possible_mappings = map_common_versionsuffixes('binutils', iccifort_binutils_tc, toolchain_mapping)
+        self.assertEqual(possible_mappings, {'-binutils-2.25': '-binutils-2.26'})
+
+        # Make sure we only map upwards, here it's gzip 1.4 in gcc and 1.6 in iccifort
+        possible_mappings = map_common_versionsuffixes('gzip', iccifort_binutils_tc, toolchain_mapping)
+        self.assertEqual(possible_mappings, {})
+
+        # newer gzip is picked up other way around (GCC -> iccifort)
+        toolchain_mapping = map_toolchain_hierarchies(gcc_binutils_tc, iccifort_binutils_tc, self.modtool)
+        possible_mappings = map_common_versionsuffixes('gzip', gcc_binutils_tc, toolchain_mapping)
+        self.assertEqual(possible_mappings, {'-gzip-1.4': '-gzip-1.6'})
+
+    def test_find_potential_version_mappings(self):
+        """Test ability to find potential version mappings of a dependency for a given toolchain mapping"""
+        test_easyconfigs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
+        init_config(build_options={
+            'robot_path': [test_easyconfigs],
+            'silent': True,
+            'valid_module_classes': module_classes(),
+        })
+        get_toolchain_hierarchy.clear()
+
+        gcc_binutils_tc = {'name': 'GCC', 'version': '4.9.3-2.26'}
+        iccifort_binutils_tc = {'name': 'iccifort', 'version': '2016.1.150-GCC-4.9.3-2.25'}
+        # The below mapping includes a binutils mapping (2.26 to 2.25)
+        tc_mapping = map_toolchain_hierarchies(gcc_binutils_tc, iccifort_binutils_tc, self.modtool)
+        ec_spec = os.path.join(test_easyconfigs, 'h', 'hwloc', 'hwloc-1.6.2-GCC-4.9.3-2.26.eb')
+        parsed_ec = process_easyconfig(ec_spec)[0]
+        gzip_dep = [dep for dep in parsed_ec['ec']['dependencies'] if dep['name'] == 'gzip'][0]
+        self.assertEqual(gzip_dep['full_mod_name'], 'gzip/1.4-GCC-4.9.3-2.26')
+
+        potential_versions = find_potential_version_mappings(gzip_dep, tc_mapping)
+        self.assertEqual(len(potential_versions), 1)
+        # Should see version 1.6 of gzip with iccifort toolchain
+        expected = {
+            'path': os.path.join(test_easyconfigs, 'g', 'gzip', 'gzip-1.6-iccifort-2016.1.150-GCC-4.9.3-2.25.eb'),
+            'toolchain': {'name': 'iccifort', 'version': '2016.1.150-GCC-4.9.3-2.25'},
+            'version': '1.6',
+        }
+        self.assertEqual(potential_versions[0], expected)
+
+    def test_map_easyconfig_to_target_tc_hierarchy(self):
+        """Test mapping of easyconfig to target hierarchy"""
+        test_easyconfigs = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'easyconfigs', 'test_ecs')
+        build_options = {
+            'robot_path': [test_easyconfigs],
+            'silent': True,
+            'valid_module_classes': module_classes(),
+        }
+        init_config(build_options=build_options)
+        get_toolchain_hierarchy.clear()
+
         gcc_binutils_tc = {'name': 'GCC', 'version': '4.9.3-2.26'}
         iccifort_binutils_tc = {'name': 'iccifort', 'version': '2016.1.150-GCC-4.9.3-2.25'}
         # The below mapping includes a binutils mapping (2.26 to 2.25)
@@ -331,6 +407,81 @@ class TweakTest(EnhancedTestCase):
             self.assertTrue(key in tweaked_dict['builddependencies'][0] and
                             value == tweaked_dict['builddependencies'][0][key])
 
+        # Now test the case where we try to update the dependencies
+        init_config(build_options=build_options)
+        get_toolchain_hierarchy.clear()
+        tweaked_spec = map_easyconfig_to_target_tc_hierarchy(ec_spec, tc_mapping, update_dep_versions=True)
+        tweaked_ec = process_easyconfig(tweaked_spec)[0]
+        tweaked_dict = tweaked_ec['ec'].asdict()
+        # First check the mapped toolchain
+        key, value = 'toolchain', iccifort_binutils_tc
+        self.assertTrue(key in tweaked_dict and value == tweaked_dict[key])
+        # Also check that binutils has been mapped
+        for key, value in {'name': 'binutils', 'version': '2.25', 'versionsuffix': ''}.items():
+            self.assertTrue(
+                key in tweaked_dict['builddependencies'][0] and value == tweaked_dict['builddependencies'][0][key]
+            )
+        # Also check that the gzip dependency was upgraded
+        for key, value in {'name': 'gzip', 'version': '1.6', 'versionsuffix': ''}.items():
+            self.assertTrue(key in tweaked_dict['dependencies'][0] and value == tweaked_dict['dependencies'][0][key])
+
+        # Make sure there are checksums for our next test
+        self.assertTrue(tweaked_dict['checksums'])
+
+        # Test the case where we also update the software version at the same time
+        init_config(build_options=build_options)
+        get_toolchain_hierarchy.clear()
+        new_version = '1.x.3'
+        tweaked_spec = map_easyconfig_to_target_tc_hierarchy(ec_spec,
+                                                             tc_mapping,
+                                                             update_build_specs={'version': new_version},
+                                                             update_dep_versions=True)
+        tweaked_ec = process_easyconfig(tweaked_spec)[0]
+        tweaked_dict = tweaked_ec['ec'].asdict()
+        # First check the mapped toolchain
+        key, value = 'toolchain', iccifort_binutils_tc
+        self.assertTrue(key in tweaked_dict and value == tweaked_dict[key])
+        # Also check that binutils has been mapped
+        for key, value in {'name': 'binutils', 'version': '2.25', 'versionsuffix': ''}.items():
+            self.assertTrue(
+                key in tweaked_dict['builddependencies'][0] and value == tweaked_dict['builddependencies'][0][key]
+            )
+        # Also check that the gzip dependency was upgraded
+        for key, value in {'name': 'gzip', 'version': '1.6', 'versionsuffix': ''}.items():
+            self.assertTrue(key in tweaked_dict['dependencies'][0] and value == tweaked_dict['dependencies'][0][key])
+
+        # Finally check that the version was upgraded
+        key, value = 'version', new_version
+        self.assertTrue(key in tweaked_dict and value == tweaked_dict[key])
+        # and that the checksum was removed
+        self.assertFalse(tweaked_dict['checksums'])
+
+        # Check that if we update a software version, it also updates the version if the software appears in an
+        # extension list (like for a PythonBundle)
+        ec_spec = os.path.join(test_easyconfigs, 't', 'toy', 'toy-0.0-gompi-2018a-test.eb')
+        # Create the trivial toolchain mapping
+        toolchain = {'name': 'gompi', 'version': '2018a'}
+        tc_mapping = map_toolchain_hierarchies(toolchain, toolchain, self.modtool)
+        # Update the software version
+        init_config(build_options=build_options)
+        get_toolchain_hierarchy.clear()
+        new_version = '1.x.3'
+        tweaked_spec = map_easyconfig_to_target_tc_hierarchy(ec_spec,
+                                                             tc_mapping,
+                                                             update_build_specs={'version': new_version},
+                                                             update_dep_versions=False)
+        tweaked_ec = process_easyconfig(tweaked_spec)[0]
+        tweaked_dict = tweaked_ec['ec'].asdict()
+        extensions = tweaked_dict['exts_list']
+        # check one extension with the same name exists and that the version has been updated
+        hit_extension = 0
+        for extension in extensions:
+            if isinstance(extension, tuple) and extension[0] == 'toy':
+                self.assertEqual(extension[1], new_version)
+                # Make sure checksum has been purged
+                self.assertFalse('checksums' in extension[2])
+                hit_extension += 1
+        self.assertEqual(hit_extension, 1, "Should only have updated one extension")
 
 def suite():
     """ return all the tests in this file """
